package com.sun.java.swing.plaf.windows;

import javax.swing.JButton;
import java.awt.GraphicsConfiguration;
import sun.awt.image.SunWritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.awt.Image;
import sun.swing.CachedPainter;
import javax.swing.SwingUtilities;
import javax.swing.CellRendererPane;
import javax.swing.JComponent;
import java.awt.Rectangle;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.border.AbstractBorder;
import javax.swing.JComboBox;
import java.awt.Graphics;
import javax.swing.text.JTextComponent;
import javax.swing.JToolBar;
import javax.swing.AbstractButton;
import javax.swing.plaf.UIResource;
import javax.swing.border.LineBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.ColorUIResource;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Component;
import javax.swing.UIManager;
import sun.awt.windows.ThemeReader;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.Toolkit;
import java.awt.Color;
import javax.swing.border.Border;
import java.util.HashMap;

class XPStyle
{
    private static XPStyle xp;
    private static XPStyle.SkinPainter skinPainter;
    private static Boolean themeActive;
    private HashMap<String, Border> borderMap;
    private HashMap<String, Color> colorMap;
    private boolean flatMenus;
    
    static synchronized void invalidateStyle() {
        XPStyle.xp = null;
        XPStyle.themeActive = null;
        XPStyle.skinPainter.flush();
    }
    
    static synchronized XPStyle getXP() {
        if (XPStyle.themeActive == null) {
            XPStyle.themeActive = (Boolean)Toolkit.getDefaultToolkit().getDesktopProperty("win.xpstyle.themeActive");
            if (XPStyle.themeActive == null) {
                XPStyle.themeActive = Boolean.FALSE;
            }
            if ((boolean)XPStyle.themeActive && AccessController.doPrivileged(new GetPropertyAction("swing.noxp")) == null && ThemeReader.isThemed() && !(UIManager.getLookAndFeel() instanceof WindowsClassicLookAndFeel)) {
                XPStyle.xp = new XPStyle();
            }
        }
        return ThemeReader.isXPStyleEnabled() ? XPStyle.xp : null;
    }
    
    static boolean isVista() {
        XPStyle xp = getXP();
        return xp != null && xp.isSkinDefined(null, TMSchema.Part.CP_DROPDOWNBUTTONRIGHT);
    }
    
    String getString(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        return getTypeEnumName(component, part, state, prop);
    }
    
    TMSchema.TypeEnum getTypeEnum(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        return TMSchema.TypeEnum.getTypeEnum(prop, ThemeReader.getEnum(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue()));
    }
    
    private static String getTypeEnumName(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        int enum1 = ThemeReader.getEnum(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
        if (enum1 == -1) {
            return null;
        }
        return TMSchema.TypeEnum.getTypeEnum(prop, enum1).getName();
    }
    
    int getInt(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop, int n) {
        return ThemeReader.getInt(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
    }
    
    Dimension getDimension(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        Dimension position = ThemeReader.getPosition(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
        return (position != null) ? position : new Dimension();
    }
    
    Point getPoint(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        Dimension position = ThemeReader.getPosition(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
        return (position != null) ? new Point(position.width, position.height) : new Point();
    }
    
    Insets getMargin(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        Insets themeMargins = ThemeReader.getThemeMargins(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
        return (themeMargins != null) ? themeMargins : new Insets(0, 0, 0, 0);
    }
    
    synchronized Color getColor(XPStyle.Skin skin, TMSchema.Prop prop, Color color) {
        String string = skin.toString() + "." + prop.name();
        TMSchema.Part part = skin.part;
        Color color2 = (Color)this.colorMap.get(string);
        if (color2 == null) {
            color2 = ThemeReader.getColor(part.getControlName(null), part.getValue(), TMSchema.State.getValue(part, skin.state), prop.getValue());
            if (color2 != null) {
                color2 = new ColorUIResource(color2);
                this.colorMap.put(string, color2);
            }
        }
        return (color2 != null) ? color2 : color;
    }
    
    Color getColor(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop, Color color) {
        return this.getColor(new XPStyle.Skin(component, part, state), prop, color);
    }
    
    synchronized Border getBorder(Component component, TMSchema.Part part) {
        if (part == TMSchema.Part.MENU) {
            if (this.flatMenus) {
                return new XPStyle.XPFillBorder(UIManager.getColor("InternalFrame.borderShadow"), 1);
            }
            return null;
        }
        else {
            XPStyle.Skin skin = new XPStyle.Skin(component, part, null);
            Border value = (Border)this.borderMap.get(skin.string);
            if (value == null) {
                String typeEnumName = getTypeEnumName(component, part, null, TMSchema.Prop.BGTYPE);
                if ("borderfill".equalsIgnoreCase(typeEnumName)) {
                    int int1 = this.getInt(component, part, null, TMSchema.Prop.BORDERSIZE, 1);
                    Color color = this.getColor(skin, TMSchema.Prop.BORDERCOLOR, Color.black);
                    value = new XPStyle.XPFillBorder(color, int1);
                    if (part == TMSchema.Part.CP_COMBOBOX) {
                        value = new XPStyle.XPStatefulFillBorder(color, int1, part, TMSchema.Prop.BORDERCOLOR);
                    }
                }
                else if ("imagefile".equalsIgnoreCase(typeEnumName)) {
                    Insets margin = this.getMargin(component, part, null, TMSchema.Prop.SIZINGMARGINS);
                    if (margin != null) {
                        if (this.getBoolean(component, part, null, TMSchema.Prop.BORDERONLY)) {
                            value = new XPStyle.XPImageBorder(component, part);
                        }
                        else if (part == TMSchema.Part.CP_COMBOBOX) {
                            value = new EmptyBorder(1, 1, 1, 1);
                        }
                        else if (part == TMSchema.Part.TP_BUTTON) {
                            value = new XPStyle.XPEmptyBorder(new Insets(3, 3, 3, 3));
                        }
                        else {
                            value = new XPStyle.XPEmptyBorder(margin);
                        }
                    }
                }
                if (value != null) {
                    this.borderMap.put(skin.string, value);
                }
            }
            return value;
        }
    }
    
    boolean isSkinDefined(Component component, TMSchema.Part part) {
        return part.getValue() == 0 || ThemeReader.isThemePartDefined(part.getControlName(component), part.getValue(), 0);
    }
    
    synchronized XPStyle.Skin getSkin(Component component, TMSchema.Part obj) {
        assert !(!this.isSkinDefined(component, obj)) : "part " + obj + " is not defined";
        return new XPStyle.Skin(component, obj, null);
    }
    
    long getThemeTransitionDuration(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.State state2, TMSchema.Prop prop) {
        return ThemeReader.getThemeTransitionDuration(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), TMSchema.State.getValue(part, state2), (prop != null) ? prop.getValue() : 0);
    }
    
    private XPStyle() {
        this.flatMenus = getSysBoolean(TMSchema.Prop.FLATMENUS);
        this.colorMap = new HashMap();
        this.borderMap = new HashMap();
    }
    
    private boolean getBoolean(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        return ThemeReader.getBoolean(part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), prop.getValue());
    }
    
    static Dimension getPartSize(TMSchema.Part part, TMSchema.State state) {
        return ThemeReader.getPartSize(part.getControlName(null), part.getValue(), TMSchema.State.getValue(part, state));
    }
    
    private static boolean getSysBoolean(TMSchema.Prop prop) {
        return ThemeReader.getSysBoolean("window", prop.getValue());
    }
    
    static String access$200(Component component, TMSchema.Part part, TMSchema.State state, TMSchema.Prop prop) {
        return getTypeEnumName(component, part, state, prop);
    }
    
    static {
        XPStyle.skinPainter = new XPStyle.SkinPainter();
        XPStyle.themeActive = null;
        invalidateStyle();
    }
    
    private class XPFillBorder extends LineBorder implements UIResource
    {
        XPFillBorder(Color color, int thickness) {
            super(color, thickness);
        }
        
        @Override
        public Insets getBorderInsets(Component component, Insets insets) {
            Insets insets2 = null;
            if ((component instanceof AbstractButton)) {
                insets2 = ((AbstractButton)component).getMargin();
            }
            else if ((component instanceof JToolBar)) {
                insets2 = ((JToolBar)component).getMargin();
            }
            else if ((component instanceof JTextComponent)) {
                insets2 = ((JTextComponent)component).getMargin();
            }
            insets.top = ((insets2 != null) ? insets2.top : 0) + this.thickness;
            insets.left = ((insets2 != null) ? insets2.left : 0) + this.thickness;
            insets.bottom = ((insets2 != null) ? insets2.bottom : 0) + this.thickness;
            insets.right = ((insets2 != null) ? insets2.right : 0) + this.thickness;
            return insets;
        }
    }
    
    private class XPStatefulFillBorder extends XPStyle.XPFillBorder
    {
        private final TMSchema.Part part;
        private final TMSchema.Prop prop;
        
        XPStatefulFillBorder(Color color, int n, TMSchema.Part part, TMSchema.Prop prop) {
            super(color, n);
            this.part = part;
            this.prop = prop;
        }
        
        @Override
        public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
            TMSchema.State state = TMSchema.State.NORMAL;
            if ((c instanceof JComboBox)) {
                JComboBox<Object> comboBox = (JComboBox)c;
                if ((comboBox.getUI() instanceof WindowsComboBoxUI)) {
                    state = ((WindowsComboBoxUI)comboBox.getUI()).getXPComboBoxState(comboBox);
                }
            }
            this.lineColor = XPStyle.this.getColor(c, this.part, state, this.prop, Color.black);
            super.paintBorder(c, g, x, y, width, height);
        }
    }
    
    private class XPImageBorder extends AbstractBorder implements UIResource
    {
        XPStyle.Skin skin;
        
        XPImageBorder(Component component, TMSchema.Part part) {
            this.skin = XPStyle.this.getSkin(component, part);
        }
        
        @Override
        public void paintBorder(Component component, Graphics graphics, int n, int n2, int n3, int n4) {
            this.skin.paintSkin(graphics, n, n2, n3, n4, null);
        }
        
        @Override
        public Insets getBorderInsets(Component component, Insets insets) {
            Insets insets2 = null;
            Insets contentMargin = this.skin.getContentMargin();
            if (contentMargin == null) {
                contentMargin = new Insets(0, 0, 0, 0);
            }
            if ((component instanceof AbstractButton)) {
                insets2 = ((AbstractButton)component).getMargin();
            }
            else if ((component instanceof JToolBar)) {
                insets2 = ((JToolBar)component).getMargin();
            }
            else if ((component instanceof JTextComponent)) {
                insets2 = ((JTextComponent)component).getMargin();
            }
            insets.top = ((insets2 != null) ? insets2.top : 0) + contentMargin.top;
            insets.left = ((insets2 != null) ? insets2.left : 0) + contentMargin.left;
            insets.bottom = ((insets2 != null) ? insets2.bottom : 0) + contentMargin.bottom;
            insets.right = ((insets2 != null) ? insets2.right : 0) + contentMargin.right;
            return insets;
        }
    }
    
    private class XPEmptyBorder extends EmptyBorder implements UIResource
    {
        XPEmptyBorder(Insets insets) {
            super(insets.top + 2, insets.left + 2, insets.bottom + 2, insets.right + 2);
        }
        
        @Override
        public Insets getBorderInsets(Component c, Insets borderInsets) {
            borderInsets = super.getBorderInsets(c, borderInsets);
            Insets insets = null;
            if ((c instanceof AbstractButton)) {
                Insets margin = ((AbstractButton)c).getMargin();
                if (c.getParent() instanceof JToolBar && !(c instanceof JRadioButton) && !(c instanceof JCheckBox) && margin instanceof InsetsUIResource) {
                    Insets insets2 = borderInsets;
                    insets2.top -= 2;
                    Insets insets3 = borderInsets;
                    insets3.left -= 2;
                    Insets insets4 = borderInsets;
                    insets4.bottom -= 2;
                    Insets insets5 = borderInsets;
                    insets5.right -= 2;
                }
                else {
                    insets = margin;
                }
            }
            else if ((c instanceof JToolBar)) {
                insets = ((JToolBar)c).getMargin();
            }
            else if ((c instanceof JTextComponent)) {
                insets = ((JTextComponent)c).getMargin();
            }
            if (insets != null) {
                borderInsets.top = insets.top + 2;
                borderInsets.left = insets.left + 2;
                borderInsets.bottom = insets.bottom + 2;
                borderInsets.right = insets.right + 2;
                return borderInsets;
            }
            return borderInsets;
        }
    }
    
    static class Skin
    {
        final Component component;
        final TMSchema.Part part;
        final TMSchema.State state;
        private final String string;
        private Dimension size;
        
        Skin(Component component, TMSchema.Part part) {
            this(component, part, null);
        }
        
        Skin(TMSchema.Part part, TMSchema.State state) {
            this(null, part, state);
        }
        
        Skin(Component component, TMSchema.Part part, TMSchema.State state) {
            this.size = null;
            this.component = component;
            this.part = part;
            this.state = state;
            String s = part.getControlName(component) + "." + part.name();
            if (state != null) {
                s = s + "(" + state.name() + ")";
            }
            this.string = s;
        }
        
        Insets getContentMargin() {
            Insets themeBackgroundContentMargins = ThemeReader.getThemeBackgroundContentMargins(this.part.getControlName(null), this.part.getValue(), 0, 100, 100);
            return (themeBackgroundContentMargins != null) ? themeBackgroundContentMargins : new Insets(0, 0, 0, 0);
        }
        
        private int getWidth(TMSchema.State state) {
            if (this.size == null) {
                this.size = XPStyle.getPartSize(this.part, state);
            }
            return (this.size != null) ? this.size.width : 0;
        }
        
        int getWidth() {
            return this.getWidth((this.state != null) ? this.state : TMSchema.State.NORMAL);
        }
        
        private int getHeight(TMSchema.State state) {
            if (this.size == null) {
                this.size = XPStyle.getPartSize(this.part, state);
            }
            return (this.size != null) ? this.size.height : 0;
        }
        
        int getHeight() {
            return this.getHeight((this.state != null) ? this.state : TMSchema.State.NORMAL);
        }
        
        @Override
        public String toString() {
            return this.string;
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof XPStyle.Skin && ((XPStyle.Skin)o).string.equals(this.string);
        }
        
        @Override
        public int hashCode() {
            return this.string.hashCode();
        }
        
        void paintSkin(Graphics graphics, int n, int n2, TMSchema.State state) {
            if (state == null) {
                state = this.state;
            }
            this.paintSkin(graphics, n, n2, this.getWidth(state), this.getHeight(state), state);
        }
        
        void paintSkin(Graphics graphics, Rectangle rectangle, TMSchema.State state) {
            this.paintSkin(graphics, rectangle.x, rectangle.y, rectangle.width, rectangle.height, state);
        }
        
        void paintSkin(Graphics graphics, int n, int n2, int n3, int n4, TMSchema.State state) {
            if (XPStyle.getXP() == null) {
                return;
            }
            if (ThemeReader.isGetThemeTransitionDurationDefined() && this.component instanceof JComponent && SwingUtilities.getAncestorOfClass(CellRendererPane.class, this.component) == null) {
                AnimationController.paintSkin((JComponent)this.component, this, graphics, n, n2, n3, n4, state);
            }
            else {
                this.paintSkinRaw(graphics, n, n2, n3, n4, state);
            }
            return;
        }
        
        void paintSkinRaw(Graphics g, int x, int y, int w, int h, TMSchema.State state) {
            if (XPStyle.getXP() == null) {
                return;
            }
            XPStyle.skinPainter.paint(null, g, x, y, w, h, new Object[] { this, state });
            return;
        }
        
        void paintSkin(Graphics g, int x, int y, int w, int h, TMSchema.State state, boolean b) {
            if (XPStyle.getXP() == null) {
                return;
            }
            if (b && "borderfill".equals(XPStyle.access$200(this.component, this.part, state, TMSchema.Prop.BGTYPE))) {
                return;
            }
            XPStyle.skinPainter.paint(null, g, x, y, w, h, new Object[] { this, state });
            return;
        }
    }
    
    private static class SkinPainter extends CachedPainter
    {
        SkinPainter() {
            super(30);
            this.flush();
        }
        
        public void flush() {
            super.flush();
        }
        
        @Override
        protected void paintToImage(Component component, Image image, Graphics graphics, int n, int n2, Object[] array) {
            XPStyle.Skin skin = (XPStyle.Skin)array[0];
            TMSchema.Part part = skin.part;
            TMSchema.State state = (TMSchema.State)array[1];
            if (state == null) {
                state = skin.state;
            }
            if (component == null) {
                component = skin.component;
            }
            DataBufferInt dataBufferInt = (DataBufferInt)((BufferedImage)image).getRaster().getDataBuffer();
            ThemeReader.paintBackground(SunWritableRaster.stealData(dataBufferInt, 0), part.getControlName(component), part.getValue(), TMSchema.State.getValue(part, state), 0, 0, n, n2, n);
            SunWritableRaster.markDirty(dataBufferInt);
        }
        
        @Override
        protected Image createImage(Component component, int width, int height, GraphicsConfiguration graphicsConfiguration, Object[] array) {
            return new BufferedImage(width, height, 2);
        }
    }
    
    static class GlyphButton extends JButton
    {
        private XPStyle.Skin skin;
        
        public GlyphButton(Component component, TMSchema.Part part) {
            XPStyle xp = XPStyle.getXP();
            this.skin = ((xp != null) ? xp.getSkin(component, part) : null);
            this.setBorder(null);
            this.setContentAreaFilled(false);
            this.setMinimumSize(new Dimension(5, 5));
            this.setPreferredSize(new Dimension(16, 16));
            this.setMaximumSize(new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
        }
        
        @Override
        public boolean isFocusTraversable() {
            return false;
        }
        
        protected TMSchema.State getState() {
            TMSchema.State state = TMSchema.State.NORMAL;
            if (!this.isEnabled()) {
                state = TMSchema.State.DISABLED;
            }
            else if (this.getModel().isPressed()) {
                state = TMSchema.State.PRESSED;
            }
            else if (this.getModel().isRollover()) {
                state = TMSchema.State.HOT;
            }
            return state;
        }
        
        public void paintComponent(Graphics graphics) {
            if (XPStyle.getXP() == null || this.skin == null) {
                return;
            }
            Dimension size = this.getSize();
            this.skin.paintSkin(graphics, 0, 0, size.width, size.height, this.getState());
            return;
        }
        
        public void setPart(Component component, TMSchema.Part part) {
            XPStyle xp = XPStyle.getXP();
            this.skin = ((xp != null) ? xp.getSkin(component, part) : null);
            this.revalidate();
            this.repaint();
        }
        
        @Override
        protected void paintBorder(Graphics graphics) {
        }
    }
}
